--------------------------------------
-- Namespaces
--------------------------------------
local AP_display_name, AP = ...
AP.UI = {};
local UI = AP.UI;
local GV = AP.GV;
local MaxXP = AP.MaxXP;

local graph;
local mainWin;
local optionWin;

local YLabels = {}
--------------------------------------
-- Functions
--------------------------------------
function UI:Debug(txt)
	if GV.debug then
		print(txt);
	end
end

function UI:GetThemeColor()
	 local c = GV.printTheme;
	 return c.r, c.g, c.b, c.hex;
end
function UI:Print(...)
	local hex = select(4, self:GetThemeColor());
    local prefix = string.format("|cff%s%s|r", hex:upper(), "Xp Chart:");
    DEFAULT_CHAT_FRAME:AddMessage(string.join(" ", prefix, ...));
	
end

	
	
function UI:Init()	
	self:Debug("UI:Init - Begin");
	----------
	
	-- Check expansion 
	local expansionLevel= GetAccountExpansionLevel();
	if expansionLevel>0 then
		UI:Print("This addon can only work with World of Warcraft Classic extension. Please disable this addon.");
		return 
	end
	
	mainWin = self:CreateMainWindow()
	optionWin = self:CreateOptionWindow()
	
	graph = self:InitGraph(mainWin)
	self:UpdateGraph()	

	----------
	self:Debug("UI:Init - End");
end

function UI:Show()
	UI:Debug("UI:Show");
	mainWin:SetShown(true);
	UI:UpdateGraph();
end
function UI:Hide()
	UI:Debug("UI:Hide");
	mainWin:SetShown(false);
end
function UI:Toggle()
	UI:Debug("UI:Toggle");
	mainWin:SetShown(not mainWin:IsShown());
	UI:UpdateGraph();
end
function UI:ShowOptions()
	UI:Debug("UI:Show");
	optionWin:SetShown(true);
end
function UI:HideOptions()
	UI:Debug("UI:Hide");
	optionWin:SetShown(false);
end
function UI:ToggleOptions()
	UI:Debug("UI:ToggleOptions");
	optionWin:SetShown(not optionWin:IsShown());
end

function UI:ShowColorPicker(r, g, b, changedCallback)
   ColorPickerFrame:SetColorRGB(r,g,b);
   ColorPickerFrame.hasOpacity = false
   ColorPickerFrame.previousValues = {r,g,b};
   ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = 
   changedCallback, changedCallback, changedCallback;
   ColorPickerFrame:Hide(); -- Need to run the OnShow handler.
   ColorPickerFrame:Show();
end

function UI:CreateOptionWindow()	
	local opt = CreateFrame("Frame","XpChart",UIParent,"BasicFrameTemplateWithInset");
	
	opt:SetSize(GV.OptionWinWidth, GV.OptionWinHeight);
	opt:SetPoint("CENTER", UIParent, "CENTER")

	opt:SetMovable(true)
	opt:EnableMouse(true)
	opt:RegisterForDrag("LeftButton")
	opt:SetScript("OnDragStart", opt.StartMoving)
	opt:SetScript("OnDragStop", opt.StopMovingOrSizing)
	
	-- Window title
	opt.title = opt:CreateFontString(nil, "OVERLAY");
	opt.title:SetFontObject("GameFontHighlight");
	opt.title:SetPoint("LEFT", opt.TitleBg, "LEFT", 5, 0);
	opt.title:SetText("Xp Chart options");

	-- Buttons
	opt.CloseOptionBtn = CreateFrame("Button", nil, opt, "GameMenuButtonTemplate" );
	opt.CloseOptionBtn:SetSize(100, 30);
	opt.CloseOptionBtn:SetPoint("BOTTOMRIGHT", opt, "BOTTOMRIGHT", -10, 10)
	opt.CloseOptionBtn:SetText("Close");
	opt.CloseOptionBtn:SetNormalFontObject("GameFontNormal");
	opt.CloseOptionBtn:SetHighlightFontObject("GameFontHighlight");
	opt.CloseOptionBtn:SetScript("OnClick", function(self, arg1) UI:HideOptions() end);

	-- Players
	opt.lblPlayers = opt:CreateFontString(nil, "OVERLAY");
	opt.lblPlayers:SetFontObject("GameFontHighlightLarge");
	opt.lblPlayers:SetPoint("TOPLEFT", opt, "TOPLEFT", 15,-40);
	opt.lblPlayers:SetText("Characters to show:");
	
	local checkbtn, colorButton;
	for i=1,#AP.DB.players do
		local pl = AP.DB.players[i]
	
		checkbtn = CreateFrame("CheckButton", nil, opt, "UICheckButtonTemplate" );
		checkbtn:SetPoint("TOPLEFT", opt, "TOPLEFT", 25,-30 + (-30 * i));
		checkbtn.text:SetText(pl.name);
		checkbtn:SetNormalFontObject("GameFontNormalLarge")
		checkbtn:SetChecked(pl.selected);
		checkbtn:SetScript("OnClick", function(self, arg1) 
				--local pName = self.text:GetText();
				--local p = AP:GetPlayer(pName);
				pl.selected = self:GetChecked();
				UI:UpdateGraph();
			end);
			
		colorButton = CreateFrame("Button", nil, opt, "GameMenuButtonTemplate")
		colorButton:SetPoint("LEFT", checkbtn, "RIGHT", 80,0)
		colorButton:SetText("Set color");
		colorButton:SetNormalFontObject("GameFontNormal");
		colorButton:SetSize(70, 25);
		-- colorButton:SetHighlightTexture("Interface\\Buttons\\CheckButtonHilight")
		colorButton:SetScript("OnClick", function(self)

						ColorPickerFrame:SetColorRGB(pl.graphLineColor[1],pl.graphLineColor[2],pl.graphLineColor[3]);
						ColorPickerFrame.hasOpacity = false
						--ColorPickerFrame.previousValues = {r,g,b};
						
						ColorPickerFrame.func = function(self) 
											--print("func"); 
										end
										
						ColorPickerFrame.opacityFunc = function(self) 
											pl.graphLineColor[1],pl.graphLineColor[2], pl.graphLineColor[3] = ColorPickerFrame:GetColorRGB() 
											pl.graphLineColor[4] = OpacitySliderFrame:GetValue()  
											UI:UpdateGraph();
											
										end
						ColorPickerFrame.cancelFunc = function(self) 
											--print("cancelFunc"); 
										end
						
						--ColorPickerFrame:Hide(); -- Need to run the OnShow handler.
						ColorPickerFrame:Show();
					end)
	end

	-- opt.resetBtn = CreateFrame("Button", nil, opt, "GameMenuButtonTemplate" );
	-- opt.resetBtn:SetSize(100, 30);
	-- opt.resetBtn:SetPoint("BOTTOMRIGHT", opt, "BOTTOMRIGHT", 20, 10)
	-- opt.resetBtn:SetText("Reset");
	-- opt.resetBtn:SetNormalFontObject("GameFontNormal");
	-- opt.resetBtn:SetHighlightFontObject("GameFontHighlight");
	--opt.resetBtn:SetNormalFontObject("GameFontNormalLarge");
	--opt.resetBtn:SetHighlightFontObject("GameFontHighlightLarge");

	-- CheckButtons
	-- opt.checkbtn1 =  CreateFrame("CheckButton", nil, opt, "UICheckButtonTemplate" );
	-- opt.checkbtn1:SetPoint("TOPRIGHT", opt, "TOPRIGHT", -100,-40);
	-- opt.checkbtn1.text:SetText("Player 1");
	-- opt.checkbtn1:SetChecked(true);
	
	opt:SetFrameLevel(mainWin:GetFrameLevel() + 10)
	opt:Hide();
	
	return opt;
end

function UI:CreateMainWindow() 
	local win = CreateFrame("Frame","XpChart",UIParent,"BasicFrameTemplateWithInset");
	
	win:SetSize(GV.MainWinWidth, GV.MainWinHeight);
	win:SetPoint("CENTER", UIParent, "CENTER")

	-- Window title
	win.title = win:CreateFontString(nil, "OVERLAY");
	win.title:SetFontObject("GameFontHighlight");
	win.title:SetPoint("LEFT", win.TitleBg, "LEFT", 5, 0);
	win.title:SetText("Xp Chart");

	-- Buttons
	win.optionsBtn = CreateFrame("Button", nil, win, "GameMenuButtonTemplate" );
	win.optionsBtn:SetSize(100, 30);
	win.optionsBtn:SetPoint("BOTTOMRIGHT", win, "BOTTOMRIGHT", -10,10)
	win.optionsBtn:SetText("Options");
	win.optionsBtn:SetNormalFontObject("GameFontNormal");
	win.optionsBtn:SetNormalFontObject("GameFontNormal");
	win.optionsBtn:SetHighlightFontObject("GameFontHighlight");
	win.optionsBtn:SetScript("OnClick", function(self, arg1) UI:ShowOptions() end);
	
	win:SetMovable(true)
	win:EnableMouse(true)
	win:RegisterForDrag("LeftButton")
	win:SetScript("OnDragStart", win.StartMoving)
	win:SetScript("OnDragStop", win.StopMovingOrSizing)

	win:SetFrameLevel(10)
	win:Hide();
	
	return win;
end

function UI:InitGraph(win) -- create win.chart
	self:Debug("UI:CreateChart - Begin");
		
	local graphLib = LibStub:GetLibrary("LibGraph-2.0");
	local g=graphLib:CreateGraphLine("XpChart-Graph",win,"TOPLEFT","TOPLEFT",	55,			-40,		win:GetWidth()-85,	win:GetHeight()-110)
	
	
	g.XGridIntervalUnit = " x"
	g.XGridIntervalMultiple = 1

	g.CreateGridlines = function(self)
			local Width=self:GetWidth()
			local Height=self:GetHeight()
			local NoSecondary=(self.GridSecondaryY==nil) or (self.GridSecondaryX==nil) or (type(self.GridColorSecondary)~="table")
			self:HideLines(self)
			self:HideFontStrings()

			if self.YGridInterval then
				local LowerYGridLine,UpperYGridLine,TopSpace
				LowerYGridLine=self.YMin/self.YGridInterval
				LowerYGridLine= math.max(math.floor(LowerYGridLine),math.ceil(LowerYGridLine))
				UpperYGridLine=self.YMax/self.YGridInterval
				UpperYGridLine=math.min(math.floor(UpperYGridLine),math.ceil(UpperYGridLine))
				TopSpace=Height*(1-(UpperYGridLine*self.YGridInterval-self.YMin)/(self.YMax-self.YMin))
				for i=LowerYGridLine,UpperYGridLine do
					if i~=0 or not self.YAxisDrawn then
						local YPos,T,F
						YPos=Height*(i*self.YGridInterval-self.YMin)/(self.YMax-self.YMin)
						if NoSecondary or math.fmod(i,self.GridSecondaryY)==0 then
							T=self:DrawLine(self,0,YPos,Width,YPos,24,self.GridColor,"BACKGROUND")
						else
							T=self:DrawLine(self,0,YPos,Width,YPos,24,self.GridColorSecondary,"BACKGROUND")
						end

						if ((i~=UpperYGridLine) or (TopSpace>12)) and (NoSecondary or math.fmod(i,self.GridSecondaryY)==0) then
							if self.YLabelsLeft then
								F=self:FindFontString()
								F:SetFontObject("GameFontHighlightSmall")
								F:SetTextColor(1,1,1)
								F:ClearAllPoints()
								F:SetPoint("BOTTOMLEFT",T,"LEFT",2,2)
								F:SetText(i*self.YGridInterval)
								F:Show()
							end

							if self.YLabelsRight then
								F=self:FindFontString()
								F:SetFontObject("GameFontHighlightSmall")
								F:SetTextColor(1,1,1)
								F:ClearAllPoints()
								F:SetPoint("BOTTOMRIGHT",T,"RIGHT",-2,2)
								F:SetText(i*self.YGridInterval)
								F:Show()
							end
						end
					end
				end
			end

			if self.XGridInterval then
				local LowerXGridLine,UpperXGridLine
				LowerXGridLine=self.XMin/self.XGridInterval
				LowerXGridLine=math.max(math.floor(LowerXGridLine),math.ceil(LowerXGridLine))
				UpperXGridLine=self.XMax/self.XGridInterval
				UpperXGridLine=math.min(math.floor(UpperXGridLine),math.ceil(UpperXGridLine))
				
				RightSpace=Width*(1-(UpperXGridLine*self.XGridInterval-self.XMin)/(self.XMax-self.XMin))

				for i=LowerXGridLine,UpperXGridLine do
					if i~=0 or not self.XAxisDrawn then
						local XPos, T
						XPos=Width*(i*self.XGridInterval-self.XMin)/(self.XMax-self.XMin)
						if NoSecondary or math.fmod(i,self.GridSecondaryX)==0 then
							T = self:DrawLine(self,XPos,0,XPos,Height,24,self.GridColor,"BACKGROUND")
						else
							T = self:DrawLine(self,XPos,0,XPos,Height,24,self.GridColorSecondary,"BACKGROUND")
						end
						
						--XPChart Tests
						if ((i~=UpperXGridLine) or (RightSpace>12)) and (NoSecondary or math.fmod(i,self.GridSecondaryX)==0) and i > 0 then
							F=self:FindFontString()
							F:SetFontObject("GameFontHighlightSmall")
							F:SetTextColor(1,1,1)
							F:ClearAllPoints()
							F:SetPoint("TOP",T,"BOTTOM",0,-5)
							F:SetText((i * self.XGridIntervalMultiple) .. self.XGridIntervalUnit)
							F:Show()
						end
					end
				end
			end

			if self.YAxisDrawn and self.YMax>=0 and self.YMin<=0 then
				local YPos,T

				YPos=Height*(-self.YMin)/(self.YMax-self.YMin)
				T=self:DrawLine(self,0,YPos,Width,YPos,24,self.AxisColor,"BACKGROUND")

				if self.YLabelsLeft  then
					F=self:FindFontString()
					F:SetFontObject("GameFontHighlightSmall")
					F:SetTextColor(1,1,1)
					F:ClearAllPoints()
					F:SetPoint("BOTTOMLEFT",T,"LEFT",2,2)
					F:SetText(0)
					F:Show()
				end
				if self.YLabelsRight  then
					F=self:FindFontString()
					F:SetFontObject("GameFontHighlightSmall")
					F:SetTextColor(1,1,1)
					F:ClearAllPoints()
					F:SetPoint("BOTTOMRIGHT",T,"RIGHT",-2,2)
					F:SetText(0)
					F:Show()
				end
			end

			if self.XAxisDrawn and self.XMax>=0 and self.XMin<=0 then
				local XPos;

				XPos=Width*(-self.XMin)/(self.XMax-self.XMin)
				self:DrawLine(self,XPos,0,XPos,Height,24,self.AxisColor,"BACKGROUND")
			end
		end

	g:SetGridColor(GV.defaultPlayedDivisionLineColor)
	g:SetAxisDrawing(false,false)
	g:SetAxisColor({1.0,1.0,1.0,1.0})
	g:SetYLabels(true, false)

	self:Debug("UI:CreateChart - End");
	
	return g;
end

function UI:UpdateGraph()

	if mainWin:IsShown() then
		local MaxWidth , MaxHeight = UI:GetChartMaxAxis();
		local SpacingWidth = 1;
		graph.XGridIntervalUnit = " sec"
		graph.XGridIntervalMultiple = 1

		if MaxWidth > 86400 then
			SpacingWidth = 86400
			graph.XGridIntervalMultiple = 1
			graph.XGridIntervalUnit = " day"
		elseif MaxWidth > 3600 then
			SpacingWidth = 3600
			graph.XGridIntervalMultiple = 1
			graph.XGridIntervalUnit = " hr"
		elseif MaxWidth > 600 then
			SpacingWidth = 600
			graph.XGridIntervalMultiple = 10
			graph.XGridIntervalUnit = " min"
		elseif MaxWidth > 60 then
			SpacingWidth = 60
			graph.XGridIntervalMultiple = 1
			graph.XGridIntervalUnit = " min"
		elseif MaxWidth > 10 then
			SpacingWidth = 10
			graph.XGridIntervalMultiple = 10
			graph.XGridIntervalUnit = " sec"
		end

		local SpacingHeight = MaxHeight * 1.5; 	
		
		graph:SetYAxis(1,MaxHeight)
		graph:SetXAxis(1,MaxWidth)	
		graph:SetGridSpacing(SpacingWidth, SpacingHeight) 	
		graph:ResetData()
		self:DrawAllLevelLines(MaxHeight, MaxWidth);
		self:DrawPlayerLines();
	end
end

function UI:DrawPlayerLines() 
	for i=1,#AP.DB.players do 
		local p = AP.DB.players[i]
		if p.selected == true then	
			if #p.points > 0 then
				--local lineColor = {1.0,1.0,1.0,1.0}
				local lineColor = p.graphLineColor;				
				local data = {}	
				for y=1,#p.points do
					data[y] = {p.points[y].played, p.points[y].xp}
				end
				graph:AddDataSeries( data , lineColor );
			end
		end
	end
end


function UI:DrawAllLevelLines(maxHeight, maxWidth) 	
	local maxLvlToShow = 1
	
	--Reset Labels
	for i=1,60,1 do	
		if YLabels[i] == nil then
			YLabels[i]= graph:CreateFontString(nil, "OVERLAY");
		end		
		YLabels[i]:ClearAllPoints()
		YLabels[i]:Hide()
	end
	
	-- Find max line
	for i=2,60,1 do	
		maxLvlToShow = i
		--print ("maxXpToShow " .. i)
		local lineLvl  = MaxXP:GetTotalMaxXp(i)
		if lineLvl > maxHeight then
			break
		end
	end
	
	--print("maxLvlToShow " .. maxLvlToShow)
	-- only show labels for the last 10 levels
	for i=2,maxLvlToShow,1 do	
	
		local lineLvl = MaxXP:GetTotalMaxXp(i -1)
		local data={{1,lineLvl},{maxWidth,lineLvl}}
		local lineColor = GV.defaultLevelDivisionLineColor
		graph:AddDataSeries(data,lineColor)

		if i==10 or i==20 or i==25 or i==30 or i==35 or i==40 or i==45 or i==50 or i==55 or (i >= maxLvlToShow - 10 ) then
		--if i==53  then
			
			--print("drawing line " .. i .. "   " .. lineLvl) 
			local YPosLbl = (lineLvl * graph:GetHeight()) / (graph.YMax-graph.YMin)	
			YLabels[i]:SetFontObject("GameFontHighlightSmall");
			YLabels[i]:SetPoint("LEFT", graph, "BOTTOMLEFT", -40, YPosLbl);
			YLabels[i]:SetText("Lvl " .. i);
			--YLabels[i]:SetText("Lvl " .. i .. " (" .. lineLvl .. ")");
			YLabels[i]:Show()
		end
	end
end


function UI:GetChartMaxAxis() 
	local maxXp = 1;
	local maxPlayed = 1;
	for i=1,#AP.DB.players do 
		local p = AP.DB.players[i]
		if p.selected == true and #p.points > 1 then
			if p.points[#p.points].played > maxPlayed then
				maxPlayed = p.points[#p.points].played
			end		
			if p.points[#p.points].xp > maxXp then
				maxXp = p.points[#p.points].xp
			end
		end
	end
	local maxXpToShow = 1;	
	for i=1,60 do 
		maxXpToShow = MaxXP:GetTotalMaxXp(i);
		--print("maxXp2 = " .. maxXp)
		--print("maxXpToShow2 = " .. maxXpToShow)
		if maxXpToShow > maxXp then
		
			break
		end
	end
	
	return maxPlayed * 1.03 , maxXpToShow * 1.03;	
end

